@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #define _XOPEN_SOURCE	/* strptime proto, but this hides other prototypes */
@@ -23,18 +23,24 @@
 #define __EXTENSIONS__	/* for solaris to get back strdup and strcasecmp */
 
 #include "config.h"
+
+#include <string.h>
+#include <stdio.h>
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+#include <stdlib.h>
+
+#include <time.h>
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+
 #include "actions.h"
 #include "i18n.h"
 #include "main.h"
 #include "version.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <stdlib.h>
-
-#include <time.h>
-#include <sys/time.h>
 
 #include <gphoto2/gphoto2-port-log.h>
 #include <gphoto2/gphoto2-setting.h>
@@ -261,7 +267,7 @@ print_info_action (GPParams *p, const char *folder, const char *filename)
 				     p->context));
 
 	printf (_("Information on file '%s' (folder '%s'):\n"),
-		filename, p->folder);
+		filename, folder);
 	printf (_("File:\n"));
 	if (info.file.fields == GP_FILE_INFO_NONE)
 		printf (_("  None available.\n"));
@@ -905,7 +911,7 @@ print_version_action (GPParams __unused__ *p)
 		  "\n"
 		  "This version of gphoto2 is using the following software versions and options:\n"),
 		VERSION,
-		2013, /* year of release! */
+		2014, /* year of release! */
 		port_message
 		);
 
@@ -1089,9 +1095,9 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 			wp.u.milliseconds	= x;
 			printf ( _("Waiting for %d milliseconds for events from camera. Press Ctrl-C to abort.\n"), x);
 		}
-		if ((arg[strlen(arg)-1]=='s') && sscanf(arg,"%ds", &x)) { /* exact seconds */ 
-			wp.type = WAIT_TIME;
-			wp.u.milliseconds = x*1000;
+		if ((wp.type != WAIT_TIME) && (arg[strlen(arg)-1]=='s') && sscanf(arg,"%ds", &x)) { /* exact seconds */ 
+			wp.type			= WAIT_TIME;
+			wp.u.milliseconds	= x*1000;
 			printf ( _("Waiting for %d seconds for events from camera. Press Ctrl-C to abort.\n"), x);
 		}
 		if (wp.type == WAIT_EVENTS) {
@@ -1340,14 +1346,15 @@ debug_func (GPLogLevel level, const char *domain, const char *str, void *data)
 	sec = tv.tv_sec  - glob_tv_zero.tv_sec;
 	usec = tv.tv_usec - glob_tv_zero.tv_usec;
 	if (usec < 0) {sec--; usec += 1000000L;}
-	fprintf (logfile, "%li.%06li %s(%i): %s\n", sec, usec, domain, level, str);
+	fprintf (logfile, "%li.%06li %-28s(%i): %s\n", sec, usec, domain, level, str);
 }
 
 int
-debug_action (GPParams *p, const char *debug_logfile_name)
+debug_action (GPParams *p, const char *debug_loglevel, const char *debug_logfile_name)
 {
 	int n;
 	FILE *logfile = NULL;
+	GPLogLevel loglevel = GP_LOG_ALL;
 
 	/* make sure we're only executed once */
 	static int debug_flag = 0;
@@ -1355,6 +1362,15 @@ debug_action (GPParams *p, const char *debug_logfile_name)
 		return(GP_OK);
 	debug_flag = 1;
 
+	if (debug_loglevel && !strcmp(debug_loglevel, "error"))
+		loglevel = GP_LOG_ERROR;
+	else if (debug_loglevel && !strcmp(debug_loglevel, "debug"))
+		loglevel = GP_LOG_DEBUG;
+	else if (debug_loglevel && !strcmp(debug_loglevel, "data"))
+		loglevel = GP_LOG_DATA;
+	else if (debug_loglevel && !strcmp(debug_loglevel, "all"))
+		loglevel = GP_LOG_ALL;
+
 	if (debug_logfile_name != NULL) {
 	  /* FIXME: Handle fopen() error besides using stderr? */
 	  logfile = fopen(debug_logfile_name, "a");
@@ -1367,7 +1383,7 @@ debug_action (GPParams *p, const char *debug_logfile_name)
 
 	gettimeofday (&glob_tv_zero, NULL);
 
-	CR (p->debug_func_id = gp_log_add_func (GP_LOG_ALL, debug_func, (void *) logfile));
+	CR (p->debug_func_id = gp_log_add_func (loglevel, debug_func, (void *) logfile));
 	gp_log (GP_LOG_DEBUG, "main", _("ALWAYS INCLUDE THE FOLLOWING LINES "
 					"WHEN SENDING DEBUG MESSAGES TO THE "
 					"MAILING LIST:"));
@@ -1613,6 +1629,7 @@ print_widget (GPParams *p, const char *name, CameraWidget *widget) {
 		printf ("Type: DATE\n");
 		printf ("Current: %d\n", t);
 		printf ("Printable: %s\n", timebuf);
+		printf ("Help: %s\n", _("Use 'now' as the current time when setting.\n"));
 		break;
 	}
 	case GP_WIDGET_MENU:
@@ -1744,19 +1761,31 @@ set_config_action (GPParams *p, const char *name, const char *value) {
 		break;
 	}
 	case GP_WIDGET_DATE:  {		/* int			*/
-		int	t = -1;
+		time_t	t = -1;
 		struct tm xtm;
 
+		memset(&xtm,0,sizeof(xtm));
+
+		/* We need to set UNIX time in seconds since Epoch */
+		/* We get ... local time */
+
+		if (!strcasecmp (value, "now")	|| !strcasecmp (value, _("now")))
+			t = time(NULL);
 #ifdef HAVE_STRPTIME
-		if (strptime (value, "%c", &xtm) || strptime (value, "%Ec", &xtm))
+		else if (strptime (value, "%c", &xtm) || strptime (value, "%Ec", &xtm)) {
+			xtm.tm_isdst = -1;
 			t = mktime (&xtm);
+		}
 #endif
 		if (t == -1) {
-			if (!sscanf (value, "%d", &t)) {
+			unsigned long lt;
+
+			if (!sscanf (value, "%ld", &lt)) {
 				gp_context_error (p->context, _("The passed value %s is neither a valid time nor an integer."), value);
 				ret = GP_ERROR_BAD_PARAMETERS;
 				break;
 			}
+			t = lt;
 		}
 		ret = gp_widget_set_value (child, &t);
 		if (ret != GP_OK)
