@@ -435,8 +435,32 @@ save_camera_file_to_file (
 		int x;
 
 		unlink(s);
-		if (-1 == rename (curname, s))
-			perror("rename");
+		if (-1 == rename (curname, s)) {
+			/* happens if the user specified a absolute path with --filename */
+			if (errno == EXDEV) {
+				char buf[8192];
+				int in_fd, out_fd;
+
+				in_fd = open(curname, O_RDONLY);
+				if (in_fd < 0)
+					perror("Can't open file for reading");
+
+				out_fd = open(s, O_CREAT | O_WRONLY, 0644);
+				if (out_fd < 0)
+					perror("Can't open file for writing");
+
+				while (1) {
+					ssize_t result = read(in_fd, buf, sizeof(buf));
+					if (!result) break;
+					if (-1 == write(out_fd, buf, result)) {
+						perror("write");
+						break;
+					}
+				}
+				unlink(curname);
+			} else
+				perror("rename");
+		}
 		x = umask(0022); /* get umask */
 		umask(x);/* set it back to the old value */
 		chmod(s,0666 & ~x);
@@ -1075,7 +1099,8 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 	 * so make sure we wait a bit for the the camera to finish stuff.
 	 */
 	gettimeofday (&expose_end_time, NULL);
-	waittime = 2000;
+	waittime = 3000;
+	printf("Capture done, waiting %gs.\n",waittime/1000.0);
 	/* tricky, this loop might need to download both JPG and RAW. so wait longer. */
 	/*if (glob_frames || end_next || !glob_interval || glob_bulblength) waittime = 2000;*/
 	/* Drain the event queue at the end and download left over added images */
@@ -1083,12 +1108,18 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 		int realwait = waittime - (-timediff_now(&expose_end_time));
 		if (realwait < 0) realwait = 0; /* just drain the queue now */
 		result = wait_and_handle_event(waittime - (-timediff_now(&expose_end_time)), &evtype, download);
-		if ((result != GP_OK) || (evtype == GP_EVENT_TIMEOUT))
+		if ((result != GP_OK) || (evtype == GP_EVENT_TIMEOUT)) {
+			/*printf("Timeout or error, leaving loop.\n");*/
 			break;
-		if (evtype == GP_EVENT_CAPTURE_COMPLETE)
+		}
+		if (evtype == GP_EVENT_CAPTURE_COMPLETE) {
+			/*printf("Capture complete, waiting final 0.1s.\n");*/
 			waittime = 100;
-		if (evtype == GP_EVENT_FILE_ADDED) /* Restart timer, more image data might come. */
+		}
+		if (evtype == GP_EVENT_FILE_ADDED) { /* Restart timer, more image data might come. */
+			/*printf("File added, waiting more %gs.\n", waittime/1000.0);*/
 			gettimeofday (&expose_end_time, NULL);
+		}
 	}
 
 	signal(SIGUSR1, SIG_DFL);
@@ -1237,6 +1268,7 @@ typedef enum {
 	ARG_CAPTURE_TETHERED_KEEP,
 	ARG_CONFIG,
 	ARG_DEBUG,
+	ARG_DEBUG_LOGLEVEL,
 	ARG_DEBUG_LOGFILE,
 	ARG_DELETE_ALL_FILES,
 	ARG_DELETE_FILE,
@@ -1892,7 +1924,7 @@ main (int argc, char **argv, char **envp)
 	poptContext ctx;
 	int i, help_option_given = 0;
 	int usage_option_given = 0;
-	char *debug_logfile_name = NULL;
+	char *debug_logfile_name = NULL, *debug_loglevel = NULL;
 	const struct poptOption generalOptions[] = {
 		GPHOTO2_POPT_CALLBACK
 		{"help", '?', POPT_ARG_NONE, (void *) &help_option_given, ARG_HELP,
@@ -1901,6 +1933,8 @@ main (int argc, char **argv, char **envp)
 		 N_("Print short message on program usage"), NULL},
 		{"debug", '\0', POPT_ARG_NONE, (void *) &debug_option_given, ARG_DEBUG,
 		 N_("Turn on debugging"), NULL},
+		{"debug-loglevel", '\0', POPT_ARG_STRING, (void *) &debug_loglevel, ARG_DEBUG_LOGLEVEL,
+		 N_("Set debug level [error|debug|data|all]"), NULL},
 		{"debug-logfile", '\0', POPT_ARG_STRING, (void *) &debug_logfile_name, ARG_DEBUG_LOGFILE,
 		 N_("Name of file to write debug info to"), N_("FILENAME")},
 		{"quiet", '\0', POPT_ARG_NONE, NULL, ARG_QUIET,
@@ -2155,7 +2189,7 @@ main (int argc, char **argv, char **envp)
 		return 0;
 	}
 	if (debug_option_given) {
-		CR_MAIN (debug_action (&gp_params, debug_logfile_name));
+		CR_MAIN (debug_action (&gp_params, debug_loglevel, debug_logfile_name));
 	}
 
 	gp_log (GP_LOG_DEBUG, "main", "invoked with following arguments:");
